/* Demo-program for Port_B on DNP/9200 */ 

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/mman.h>
#include <signal.h>

typedef unsigned char  u8;
typedef unsigned short u16;
typedef unsigned int   u32;

void *pMapmemory(off_t phy_addr, size_t phy_lenght);

volatile u32 *pPIO_BASE;

/* PIOA Header */
#define PIOA_PER   (*(pPIO_BASE + 0x400 / 4)) /* PIOA Enable Register */
#define PIOA_PDR   (*(pPIO_BASE + 0x404 / 4)) /* PIOA Disable Register */
#define PIOA_OER   (*(pPIO_BASE + 0x410 / 4)) /* Output Enable Register (output) */
#define PIOA_ODR   (*(pPIO_BASE + 0x414 / 4)) /* Output Disable Register (input) */
#define PIOA_SODR  (*(pPIO_BASE + 0x430 / 4)) /* Set Output Data Register (outset) */
#define PIOA_CODR  (*(pPIO_BASE + 0x434 / 4)) /* Clear Output Data Register (outclr) */
#define PIOA_ODSR  (*(pPIO_BASE + 0x438 / 4)) /* Output Data Status Register (read output) */
#define PIOA_PDSR  (*(pPIO_BASE + 0x43C / 4)) /* Pin Data Status Register (read input) */

/* PIOB Header */
#define PIOB_PER   (*(pPIO_BASE + 0x600 / 4)) /* PIOB Enable Register */
#define PIOB_PDR   (*(pPIO_BASE + 0x604 / 4)) /* PIOB Disable Register */
#define PIOB_OER   (*(pPIO_BASE + 0x610 / 4)) /* Output Enable Register (output) */
#define PIOB_ODR   (*(pPIO_BASE + 0x614 / 4)) /* Output Disable Register (input) */
#define PIOB_SODR  (*(pPIO_BASE + 0x630 / 4)) /* Set Output Data Register (outset) */
#define PIOB_CODR  (*(pPIO_BASE + 0x634 / 4)) /* Clear Output Data Register (outclr) */
#define PIOB_ODSR  (*(pPIO_BASE + 0x638 / 4)) /* Output Data Status Register (read output) */
#define PIOB_PDSR  (*(pPIO_BASE + 0x63C / 4)) /* Pin Data Status Register (read input) */

/* PIOC Header */
#define PIOC_PER   (*(pPIO_BASE + 0x800 / 4)) /* PIOC Enable Register */
#define PIOC_PDR   (*(pPIO_BASE + 0x804 / 4)) /* PIOC Disable Register */
#define PIOC_OER   (*(pPIO_BASE + 0x810 / 4)) /* Output Enable Register (output) */
#define PIOC_ODR   (*(pPIO_BASE + 0x814 / 4)) /* Output Disable Register (input) */
#define PIOC_SODR  (*(pPIO_BASE + 0x830 / 4)) /* Set Output Data Register (outset) */
#define PIOC_CODR  (*(pPIO_BASE + 0x834 / 4)) /* Clear Output Data Register (outclr) */
#define PIOC_ODSR  (*(pPIO_BASE + 0x838 / 4)) /* Output Data Status Register (read output) */
#define PIOC_PDSR  (*(pPIO_BASE + 0x83C / 4)) /* Pin Data Status Register (read input) */

/* PIOD Header */
#define PIOD_PER   (*(pPIO_BASE + 0xA00 / 4)) /* PIOD Enable Register */
#define PIOD_PDR   (*(pPIO_BASE + 0xA04 / 4)) /* PIOD Disable Register */
#define PIOD_OER   (*(pPIO_BASE + 0xA10 / 4)) /* Output Enable Register (output) */
#define PIOD_ODR   (*(pPIO_BASE + 0xA14 / 4)) /* Output Disable Register (input) */
#define PIOD_SODR  (*(pPIO_BASE + 0xA30 / 4)) /* Set Output Data Register (outset) */
#define PIOD_CODR  (*(pPIO_BASE + 0xA34 / 4)) /* Clear Output Data Register (outclr) */
#define PIOD_ODSR  (*(pPIO_BASE + 0xA38 / 4)) /* Output Data Status Register (read output) */
#define PIOD_PDSR  (*(pPIO_BASE + 0xA3C / 4)) /* Pin Data Status Register (read input) */


int init_mem (void)
{
    /* map AT91 PIO register space at 0xFFFFFF000 / size 4096 bytes */
    pPIO_BASE = pMapmemory(0xFFFFF000, 0x00001000);
    if (!pPIO_BASE) {
    printf("MMAP PIO_BASE fail !\n");
    exit(1);
    }
	return(0);
}


void *pMapmemory(off_t phy_addr, size_t phy_lenght)
{
#define MAP_PAGESIZE 4096UL

int iFd;
void *pMem;

    if ((phy_addr % MAP_PAGESIZE) != 0) {
    fprintf(stderr,"physical address not aligned on PAGESIZE boundary!\n");
    return(NULL);
    }

    if ((phy_lenght % MAP_PAGESIZE) != 0) {
    fprintf(stderr,"physical lenght not aligned on PAGESIZE boundary!\n");
    return(NULL);
    }

    /* open mem device for read/write */
    iFd = open("/dev/mem", O_RDWR | O_SYNC);
    if (iFd < 0) {
    fprintf(stderr,"open of /dev/mem fail !\n");
    return(NULL);
    }

    /* get pointer to DNP9200 memory */
    pMem = mmap(NULL,
        phy_lenght,
        (PROT_READ | PROT_WRITE),
        MAP_SHARED,
        iFd,
        phy_addr);

    if ((pMem == MAP_FAILED) || (pMem == NULL)) {
    fprintf(stderr,"mmap of /dev/mem fail !\n");
    return(NULL);
    }

    /* close mem device */
    if (close(iFd) != 0)
    fprintf(stderr,"close of /dev/mem fail !\n");

    return(pMem);
}

int main (int argc, char *argv[])
{

    /* check user identity */
    if (geteuid() != 0) {
    printf("No root access rights !\n");
    exit(1);
    }

    init_mem();

    if (argc < 2) {
        printf("Usage 1=on 0=off");
        return 0;
    }
    
    PIOD_PER = (0x00000002); //enable pio
    PIOD_OER = (0x00000002); //enable output
    
    if (strcmp(argv[1],"1")==0) {
        printf("Modem an");
        PIOD_SODR = (0x00000002); //set
        sleep(2);
        PIOD_CODR = (0x00000002); //clear
        usleep(500000);
        PIOD_SODR = (0x00000002); // set
    }else if(strcmp(argv[1],"0")==0){
        printf("Modem aus");
        PIOD_SODR = (0x00000002); //set
        sleep(2);
        PIOD_CODR = (0x00000002); //clear
        sleep(2);
        PIOD_SODR = (0x00000002); // set
    }
    
    return 0;
}